@@ -50,18 +50,15 @@
  *
  * Gregory P. Smith <greg@krypto.org> is once again the maintainer.
  *
- * Use the pybsddb-users@lists.sf.net mailing list for all questions.
- * Things can change faster than the header of this file is updated.  This
- * file is shared with the PyBSDDB project at SourceForge:
+ * Since January 2008, new maintainer is Jesus Cea <jcea@jcea.es>.
+ * Jesus Cea licenses this code to PSF under a Contributor Agreement.
  *
- * http://pybsddb.sf.net
+ * Use the pybsddb@jcea.es mailing list for all questions.
+ * Things can change faster than the header of this file is updated.
  *
- * This file should remain backward compatible with Python 2.1, but see PEP
- * 291 for the most current backward compatibility requirements:
+ * http://www.jcea.es/programacion/pybsddb.htm
  *
- * http://www.python.org/peps/pep-0291.html
- *
- * This module contains 6 types:
+ * This module contains 8 types:
  *
  * DB           (Database)
  * DBCursor     (Database Cursor)
@@ -69,9 +66,7 @@
  * DBTxn        (An explicit database transaction)
  * DBLock       (A lock handle)
  * DBSequence   (Sequence)
- *
- * New datatypes:
- *
+ * DBSite       (Site)
  * DBLogCursor  (Log Cursor)
  *
  */
@@ -109,7 +104,7 @@
 #error "eek! DBVER can't handle minor versions > 9"
 #endif
 
-#define PY_BSDDB_VERSION "5.1.0"
+#define PY_BSDDB_VERSION "6.2.1"
 
 /* Python object definitions */
 
@@ -129,6 +124,9 @@ struct DBCursorObject;    /* Forward declaration */
 struct DBLogCursorObject; /* Forward declaration */
 struct DBTxnObject;       /* Forward declaration */
 struct DBSequenceObject;  /* Forward declaration */
+#if (DBVER >= 53)
+struct DBSiteObject;      /* Forward declaration */
+#endif
 
 typedef struct {
     PyObject_HEAD
@@ -140,6 +138,9 @@ typedef struct {
     struct DBObject *children_dbs;
     struct DBTxnObject *children_txns;
     struct DBLogCursorObject *children_logcursors;
+#if (DBVER >= 53)
+    struct DBSiteObject *children_sites;
+#endif
     PyObject        *private_obj;
     PyObject        *rep_transport;
     PyObject        *in_weakreflist; /* List of weak references */
@@ -154,15 +155,14 @@ typedef struct DBObject {
     struct behaviourFlags moduleFlags;
     struct DBTxnObject *txn;
     struct DBCursorObject *children_cursors;
-#if (DBVER >=43)
     struct DBSequenceObject *children_sequences;
-#endif
     struct DBObject **sibling_prev_p;
     struct DBObject *sibling_next;
     struct DBObject **sibling_prev_p_txn;
     struct DBObject *sibling_next_txn;
     PyObject*       associateCallback;
     PyObject*       btCompareCallback;
+    PyObject*       dupCompareCallback;	    
     int             primaryDBType;
     PyObject        *private_obj;
     PyObject        *in_weakreflist; /* List of weak references */
@@ -207,6 +207,16 @@ typedef struct DBLogCursorObject {
     PyObject        *in_weakreflist; /* List of weak references */
 } DBLogCursorObject;
 
+#if (DBVER >= 53)
+typedef struct DBSiteObject {
+    PyObject_HEAD
+    DB_SITE         *site;
+    DBEnvObject     *env;
+    struct DBSiteObject **sibling_prev_p;
+    struct DBSiteObject *sibling_next;
+    PyObject    *in_weakreflist; /* List of weak references */
+} DBSiteObject;
+#endif
 
 typedef struct {
     PyObject_HEAD
@@ -216,7 +226,6 @@ typedef struct {
 } DBLockObject;
 
 
-#if (DBVER >= 43)
 typedef struct DBSequenceObject {
     PyObject_HEAD
     DB_SEQUENCE*     sequence;
@@ -228,7 +237,6 @@ typedef struct DBSequenceObject {
     struct DBSequenceObject *sibling_next_txn;
     PyObject        *in_weakreflist; /* List of weak references */
 } DBSequenceObject;
-#endif
 
 
 /* API structure for use by C code */
@@ -241,10 +249,9 @@ typedef struct DBSequenceObject {
      PyObject*  mod;
      PyObject*  cobj;
 
-     mod  = PyImport_ImportModule("bsddb._bsddb");
-     // Use "bsddb3._pybsddb" if you're using the standalone pybsddb add-on.
+     mod  = PyImport_ImportModule("bsddb3._pybsddb");
      cobj = PyObject_GetAttrString(mod, "api");
-     api  = (BSDDB_api*)PyCObject_AsVoidPtr(cobj);
+     bsddb_api  = (BSDDB_api*)PyCObject_AsVoidPtr(cobj);
      Py_DECREF(cobj);
      Py_DECREF(mod);
 
@@ -252,9 +259,7 @@ typedef struct DBSequenceObject {
      // If you are using Python 2.7 or up: (except Python 3.0, unsupported)
      BSDDB_api* bsddb_api;
 
-     // Use "bsddb3._pybsddb.api" if you're using
-     // the standalone pybsddb add-on.
-     bsddb_api = (void **)PyCapsule_Import("bsddb._bsddb.api", 1);
+     bsddb_api = (void **)PyCapsule_Import("bsddb3._pybsddb.api", 1);
 
 
    Check "api_version" number before trying to use the API.
@@ -272,7 +277,7 @@ typedef struct {
     PyTypeObject* dbenv_type;
     PyTypeObject* dbtxn_type;
     PyTypeObject* dblock_type;
-    PyTypeObject* dbsequence_type;  /* If DBVER < 43 -> NULL */
+    PyTypeObject* dbsequence_type;
 
     /* Functions */
     int (*makeDBError)(int err);
