@@ -1,12 +1,86 @@
 #!/usr/bin/env python
-#----------------------------------------------------------------------
-# Setup script for the bsddb3 package
+
+"""
+Copyright (c) 2008-2016, Jesus Cea Avion <jcea@jcea.es>
+All rights reserved.
+
+Redistribution and use in source and binary forms, with or without
+modification, are permitted provided that the following conditions
+are met:
+
+1. Redistributions of source code must retain the above copyright
+notice, this list of conditions and the following disclaimer.
+
+2. Redistributions in binary form must reproduce the above
+copyright notice, this list of conditions and the following
+disclaimer in the documentation and/or other materials provided
+with the distribution.
+
+3. Neither the name of Jesus Cea Avion nor the names of its
+contributors may be used to endorse or promote products derived
+from this software without specific prior written permission.
+
+THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
+CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
+INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
+MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
+BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
+TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
+ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
+TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
+THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+SUCH DAMAGE.
+"""
 
 import os
 import re
 import sys
 import glob
 
+if (sys.version_info[0] < 3) and (sys.version_info >= (2, 6)) :
+    # Silence deprecation warnings during "setup"
+    import warnings
+    warnings.filterwarnings('ignore',
+            message='in 3.x, bsddb has been removed; ' \
+                    'please use the pybsddb project instead',
+            category=DeprecationWarning)
+    warnings.filterwarnings('ignore',
+            message='in 3.x, the bsddb module has been removed; ' \
+                    'please use the pybsddb project instead',
+            category=DeprecationWarning)
+
+    if sys.version_info[:2] == (2, 7) :
+        with warnings.catch_warnings() :
+            # Python 2.7.0
+            warnings.filterwarnings('ignore',
+                message='The CObject type is marked Pending Deprecation ' \
+                        'in Python 2.7.  Please use capsule objects instead.',
+                category=PendingDeprecationWarning)
+
+            # Python 2.7.1
+            warnings.filterwarnings('ignore',
+                message='CObject type is not supported in 3.x. ' \
+                        'Please use capsule objects instead.',
+                category=DeprecationWarning)
+
+            try :  # 'bsddb' library could be not available
+                import bsddb  # Import the 2.7 version, that uses CObject
+            except ImportError :
+                pass
+
+    # setuptools warnings
+    warnings.filterwarnings('ignore',
+            message='tuple parameter unpacking has been removed in 3.x',
+            category=SyntaxWarning)
+    warnings.filterwarnings('ignore',
+            message='the dl module has been removed in Python 3.0; ' \
+                    'use the ctypes module instead',
+            category=DeprecationWarning)
+
+
 try :
   from setuptools import setup, Extension
 except :
@@ -28,6 +102,9 @@ for line in _srcFile.readlines():
     if m:
         VERSION = m.group(1)
         continue
+
+# We close first, to avoid a "ResourceWarning"
+_srcFile.close()
 del _srcFile
 del _ver_re
 del m
@@ -74,33 +151,46 @@ if os.name == 'posix':
     if LFLAGS or LIBS:
         lflags_arg = LFLAGS + LIBS
 
+    # Supported Berkeley DB versions, in order of preference.
+    db_ver_list = ((6, 2), (6, 1),
+            (5, 3), (5, 1),
+            (4, 8), (4, 7))
+    db_ver = None
+
     # If we were not told where it is, go looking for it.
     dblib = 'db'
     incdir = libdir = None
     if not BERKELEYDB_DIR and not BERKELEYDB_LIBDIR and not BERKELEYDB_INCDIR:
-        # NOTE: when updating these, also change the tuples in the for loops below
-        max_db_ver = (4, 8)
-        min_db_ver = (4, 1)
-
         # construct a list of paths to look for the header file in on
         # top of the normal inc_dirs.
-        db_inc_paths = [
-            '/usr/include/db4',
-            '/usr/local/include/db4',
-            '/opt/sfw/include/db4',
-            '/sw/include/db4',
-            '/usr/include/db3',
-            '/usr/local/include/db3',
-            '/opt/sfw/include/db3',
-            '/sw/include/db3',
-        ]
-        # 4.x minor number specific paths
-        for x in range(max_db_ver[1]+1):
-            db_inc_paths.append('/usr/include/db4%d' % x)
-            db_inc_paths.append('/usr/local/BerkeleyDB.4.%d/include' % x)
-            db_inc_paths.append('/usr/local/include/db4%d' % x)
-            db_inc_paths.append('/pkg/db-4.%d/include' % x)
-            db_inc_paths.append('/opt/db-4.%d/include' % x)
+        db_inc_paths = []
+        db_major = set()
+        bitness = ""
+        import platform
+        if (platform.architecture()[0] == "64bit") and \
+           (platform.platform(True, True).startswith("Solaris-")) :
+               bitness = "/64"
+
+        for major, minor in db_ver_list :
+            if major not in db_major :
+                db_major.add(major)
+                db_inc_paths.extend([
+                    '/usr/include/db%d%s' %(major, bitness),
+                    '/usr/local/include/db%d%s' %(major, bitness),
+                    '/opt/sfw/include/db%d%s' %(major, bitness),
+                    '/sw/include/db%d%s' %(major, bitness),
+                    ])
+
+            db_inc_paths.append('/usr/include/db%d%d%s' % \
+                (major, minor, bitness))
+            db_inc_paths.append('/usr/local/BerkeleyDB.%d.%d%s/include' % \
+                (major, minor, bitness))
+            db_inc_paths.append('/usr/local/include/db%d%d%s' % \
+                (major, minor, bitness))
+            db_inc_paths.append('/pkg/db-%d.%d%s/include' % \
+                (major, minor, bitness))
+            db_inc_paths.append('/opt/db-%d.%d%s/include' % \
+                (major, minor, bitness))
 
         db_ver_inc_map = {}
 
@@ -112,6 +202,7 @@ if os.name == 'posix':
             lib_dirs = compiler.library_dirs + [
                 '/lib64', '/usr/lib64',
                 '/lib', '/usr/lib',
+                '/usr/lib/x86_64-linux-gnu/',
             ]
             inc_dirs = compiler.include_dirs + ['/usr/include']
 
@@ -121,7 +212,8 @@ if os.name == 'posix':
                 f = os.path.join(d, "db.h")
                 if debug: print("db: looking for db.h in", f)
                 if os.path.exists(f):
-                    f = open(f).read()
+                    with open(f) as fichero :
+                        f = fichero.read()
                     m = re.search(r"#define\WDB_VERSION_MAJOR\W(\d+)", f)
                     if m:
                         db_major = int(m.group(1))
@@ -130,7 +222,7 @@ if os.name == 'posix':
                         db_ver = (db_major, db_minor)
 
                         if ( (db_ver not in db_ver_inc_map) and
-                           (db_ver <= max_db_ver and db_ver >= min_db_ver) ):
+                           (db_ver in db_ver_list) ):
                             # save the include directory with the db.h version
                             # (first occurrance only)
                             db_ver_inc_map[db_ver] = d
@@ -207,6 +299,7 @@ if os.name == 'posix':
     # Test if the old bsddb is built-in
     static = 0
     try:
+        # Possibly already imported in the "warning" section (python 2.7)
         import bsddb
         if str(bsddb).find('built-in') >= 0:
             static = 1
@@ -234,6 +327,28 @@ if os.name == 'posix':
         if st != "yes":
             sys.exit(1)
 
+    # read db.h to figure out what version of Berkeley DB this is
+    ver = None
+    with open(os.path.join(incdir, 'db.h'), 'r') as f :
+        db_h_lines = f.readlines()
+    db_ver_re = re.compile(
+        r'^#define\s+DB_VERSION_STRING\s.*Berkeley DB (\d+\.\d+).*')
+    db_ver2 = db_ver
+    if db_ver is None :
+        print("Trying to use the Berkeley DB you specified...")
+    for line in db_h_lines:
+        match = db_ver_re.match(line)
+        if not match:
+            continue
+        fullverstr = match.group(1)
+        ver = fullverstr[0] + fullverstr[2]   # 31 == 3.1, 32 == 3.2, etc.
+        db_ver = (int(fullverstr[0]), int(fullverstr[2]))
+    if (db_ver2 is not None) and (db_ver != db_ver2) :
+        raise AssertionError("Detected Berkeley DB version is inconsistent")
+    if db_ver not in db_ver_list:
+        raise AssertionError("pybsddb untested with this Berkeley DB "
+                "version %d.%d" %db_ver)
+    print('Detected Berkeley DB version %d.%d from db.h' %db_ver)
 
 elif os.name == 'nt':
 
@@ -245,7 +360,7 @@ elif os.name == 'nt':
     # version numbers in default path names, and different library
     # names for debug/release or dll/static.
     #
-    # So to make things easier, I'm just going to exepect that the DB stuff
+    # So to make things easier, I'm just going to expect that the DB stuff
     # has been moved to the ./db directory. There's an updatedb.bat file to
     # help.
     #
@@ -258,7 +373,8 @@ elif os.name == 'nt':
 
     # read db.h to figure out what version of Berkeley DB this is
     ver = None
-    db_h_lines = open(os.path.join(incdir, 'db.h'), 'r').readlines()
+    with open(os.path.join(incdir, 'db.h'), 'r') as f :
+        db_h_lines = f.readlines()
     db_ver_re = re.compile(
         r'^#define\s+DB_VERSION_STRING\s.*Berkeley DB (\d+\.\d+).*')
     for line in db_h_lines:
@@ -267,9 +383,11 @@ elif os.name == 'nt':
             continue
         fullverstr = match.group(1)
         ver = fullverstr[0] + fullverstr[2]   # 31 == 3.1, 32 == 3.2, etc.
-    assert ver in ('41', '42', '43', '44', '45', '46', '47', '48'), (
-        "pybsddb untested with this Berkeley DB version", ver)
-    print('Detected Berkeley DB version', ver, 'from db.h')
+        db_ver = (int(fullverstr[0]), int(fullverstr[2]))
+    if db_ver not in db_ver_list:
+        raise AssertionError("pybsddb untested with this Berkeley DB "
+                "version %d.%d" %db_ver)
+    print('Detected Berkeley DB version %d.%d from db.h' %db_ver)
 
     if debug:
         libname = ['libdb%ssd' % ver]     # Debug, static
@@ -291,6 +409,45 @@ elif os.name == 'nt':
              ("bsddb3/test", glob.glob("test/*.py"))
              ]
 
+if (db_ver in ((6, 0), (6, 1), (6, 2))) and \
+  ("YES_I_HAVE_THE_RIGHT_TO_USE_THIS_BERKELEY_DB_VERSION" not in os.environ) :
+    print (
+        "\n"
+        "******* COMPILATION ABORTED *******\n"
+        "\n"
+        "You are linking a Berkeley DB version licensed under "
+        "AGPL3 or have a commercial license.\n"
+        "\n"
+        "AGPL3 is a strong copyleft license and derivative "
+        "works must be equivalently licensed.\n"
+        "\n"
+        "You have two choices:\n"
+        "\n"
+        "  1. If your code is AGPL3 or you have a commercial Berkeley DB "
+        "license from Oracle, please, define the environment variable "
+        "'YES_I_HAVE_THE_RIGHT_TO_USE_THIS_BERKELEY_DB_VERSION' to "
+        "any value, and try to install this python library again.\n"
+        "\n"
+        "  2. In any other case, you have to link to a previous version "
+        "of Berkeley DB. Remove Berlekey DB version 6.x and let this "
+        "python library try to locate an older version of the "
+        "Berkeley DB library in your system. Alternatively, you can "
+        "define the environment variable 'BERKELEYDB_DIR', or "
+        "'BERKELEYDB_INCDIR' and 'BERKELEYDB_LIBDIR', with the path of "
+        "the Berkeley DB you want to use and try to install this "
+        "python library again.\n"
+        "\n"
+        "Sorry for the inconvenience. I am trying to protect you.\n"
+        "\n"
+        "More details:\n"
+        "\n"
+        "    https://forums.oracle.com/message/11184885\n"
+        "    http://lists.debian.org/debian-legal/2013/07/\n"
+        "\n"
+        "******* COMPILATION ABORTED *******\n"
+        )
+    sys.exit(1)
+
 version_suffix = ""
 if sys.version_info[0] > 2 :
     version_suffix = "3"
@@ -298,37 +455,37 @@ if sys.version_info[0] > 2 :
 # do the actual build, install, whatever...
 setup(name = 'bsddb3',
       version = VERSION,
-      description = 'Python interface for Berkeley DB',
+      description = 'Python bindings for Oracle Berkeley DB',
       long_description = """\
-This module provides a nearly complete wrapping
-of the Oracle/Sleepycat C API for the Database
-Environment, Database, Cursor, Sequence and Transaction
-objects, and each of these is exposed as a Python
-type in the bsddb3.db module. The database objects
-can use various access methods: btree, hash, recno,
-and queue. Complete support of Berkeley DB distributed
-transactions. Complete support for Berkeley DB
-Replication Manager. Complete support for Berkeley DB
-Base Replication. Support for RPC.
+This module provides a nearly complete wrapping of the Oracle/Sleepycat C API
+for the Database Environment, Database, Cursor, Log Cursor, Sequence and
+Transaction objects, and each of these is exposed as a Python type in the
+bsddb3.db module. The database objects can use various access methods: btree,
+hash, recno, and queue.  Complete support of Berkeley DB distributed
+transactions. Complete support for Berkeley DB Replication Manager. Complete
+support for Berkeley DB Base Replication. Support for RPC.
 
-Please see the documents in the docs
-directory of the source distribution or at the
-website for more details on the types and methods
-provided. The goal is to mirror most of the real
-Berkeley DB API so fall back to the Oracle Berkeley DB
+Please see the documents in the docs directory of the source distribution or at
+the website for more details on the types and methods provided. The goal is to
+mirror most of the real Berkeley DB API so fall back to the Oracle Berkeley DB
 documentation as appropriate.
 
+If you need to support ancient versiones of Python and/or Berkeley DB , you can
+use old releases of this bindings.
+
+
 `Homepage <http://www.jcea.es/programacion/
 pybsddb.htm>`__ --
 `Documentation <http://www.jcea.es/programacion/
 pybsddb_doc/>`__ --
-`Mailing List <http://mailman.jcea.es/listinfo/pybsddb>`__
+`Mailing List <https://mailman.jcea.es/listinfo/pybsddb>`__ --
+`Donation <http://www.jcea.es/programacion/pybsddb_doc/donate.html>`__
 """,
 
       author = 'Jesus Cea, Robin Dunn, Gregory P. Smith, Andrew Kuchling, Barry Warsaw',
       author_email = 'pybsddb@jcea.es',
       url = 'http://www.jcea.es/programacion/pybsddb.htm',
-      license = "MIT License",
+      license = "3-clause BSD License",
 
       packages = ['bsddb3', 'bsddb3/tests'],
       package_dir = {'bsddb3': 'Lib%s/bsddb' %version_suffix,
@@ -337,7 +494,6 @@ pybsddb_doc/>`__ --
                                sources = ['Modules/_bsddb.c'],
                                depends = ['Modules/bsddb.h'],
                                include_dirs = [ incdir ],
-                               define_macros = [('PYBSDDB_STANDALONE', 1)],
                                library_dirs = [ libdir ],
                                runtime_library_dirs = [ libdir ],
                                libraries = libname,
@@ -346,7 +502,7 @@ pybsddb_doc/>`__ --
       headers = ['Modules/bsddb.h'],
       data_files = utils,
       classifiers = [
-                    'License :: OSI Approved :: MIT License',
+                    'License :: OSI Approved :: BSD License',
                     'Development Status :: 5 - Production/Stable',
                     'Intended Audience :: Developers',
                     'Intended Audience :: Information Technology',
@@ -358,14 +514,12 @@ pybsddb_doc/>`__ --
                     'Topic :: Software Development',
                     'Topic :: System :: Clustering',
                     'Programming Language :: Python :: 2',
-                    'Programming Language :: Python :: 2.3',
-                    'Programming Language :: Python :: 2.4',
-                    'Programming Language :: Python :: 2.5',
                     'Programming Language :: Python :: 2.6',
                     'Programming Language :: Python :: 2.7',
                     'Programming Language :: Python :: 3',
-                    'Programming Language :: Python :: 3.0',
-                    'Programming Language :: Python :: 3.1',
-                    'Programming Language :: Python :: 3.2',
+                    'Programming Language :: Python :: 3.3',
+                    'Programming Language :: Python :: 3.4',
+                    'Programming Language :: Python :: 3.5',
                     ],
       )
+
